import nltk  # type: ignore
import matplotlib.pyplot as plt
from chatInsights import _transformData
from wordcloud import WordCloud  # type: ignore
from nltk.corpus import stopwords  # type: ignore


class ChatInsights:
    def __init__(self, file_path: str) -> None:
        self.file_path = file_path
        self.data = _transformData(self.file_path)

    def active_users(self, save_figure: str = "N") -> None:
        """
        Generate a bar chart of the most active users in the group.

        Parameters:
        -----------
        save_figure : str, optional
            Specifies whether to save the figure or not. 
            "Y" to save the figure, "N" to not save. 
            Default is "N".

        Returns:
        --------
        None
        """
        
        # function implementation
        plt.figure(figsize=(8, 4))
        m_a = self.data["user"].value_counts().head(10).sort_values()
        m_a.plot.barh(
            color=[
                "#696969",
                "#0ff1ce",
                "#ffd700",
                "#ffff00",
                "#ff0000",
                "#ac25e2",
                "#ff00ff",
                "#00ced1",
                "#008080",
                "#8cff32",
            ]
        )

        for index, value in enumerate(m_a):
            plt.text(
                value + 3, index, str(value), va="center", color="black", fontsize=7
            )

        plt.ylabel("Users")
        plt.xlabel("No. of messages")
        plt.title("Users Activity Stats", fontdict={"fontsize": 11})
        if save_figure == "Y" or save_figure == "y":
            plt.savefig("active_users.png", bbox_inches="tight")
        plt.show()

    def active_day(self, save_figure: str = "N") -> None:
        """
        Generate a bar chart show users activity day-wise..

        Parameters:
        -----------
        save_figure : str, optional
            Specifies whether to save the figure or not. 
            "Y" to save the figure, "N" to not save. 
            Default is "N".

        Returns:
        --------
        None
        """
        
        # function implementation
        plt.figure(figsize=(7, 3))
        a_d = self.data["day"].value_counts().head(10).sort_values()
        a_d.plot.barh(
            color=[
                "#8cff32",
                "#0ff1ce",
                "#ffd700",
                "#ffff00",
                "#ff0000",
                "#ac25e2",
                "#ff00ff",
            ]
        )

        for index, value in enumerate(a_d):
            plt.text(
                value + 3, index, str(value), va="center", color="black", fontsize=6
            )

        plt.ylabel("Week Days")
        plt.xlabel("No. of messages")
        plt.title("Weekly Message Stats", fontdict={"fontsize": 11})
        if save_figure == "Y" or save_figure == "y":
            plt.savefig("active_day.png", bbox_inches="tight")
        plt.show()

    def active_month(self, save_figure: str = "N") -> None:
        """
        Generate a bar chart show users activity month-wise.

        Parameters:
        -----------
        save_figure : str, optional
            Specifies whether to save the figure or not. 
            "Y" to save the figure, "N" to not save. 
            Default is "N".

        Returns:
        --------
        None
        """
        
        # function implementation
        plt.figure(figsize=(8, 4))
        a_m = self.data["month_year"].value_counts().sort_values()[:10]
        a_m.plot.barh(
            color=[
                "#8cff32",
                "#0ff1ce",
                "#ffd700",
                "#ffff00",
                "#ff0000",
                "#ac25e2",
                "#ff00ff",
                "#00ced1",
                "#008080",
                "#696969",
            ]
        )

        for index, value in enumerate(a_m):
            plt.text(
                value + 3, index, str(value), va="center", color="black", fontsize=7
            )

        plt.ylabel("Months")
        plt.xlabel("No. of messages")
        plt.title("Monthly Message Stats", fontdict={"fontsize": 11})
        if save_figure == "Y" or save_figure == "y":
            plt.savefig("active_month.png", bbox_inches="tight")
        plt.show()

    def active_year(self, save_figure: str = "N") -> None:
        """
        Generate a bar chart show users activity year-wise.

        Parameters:
        -----------
        save_figure : str, optional
            Specifies whether to save the figure or not. 
            "Y" to save the figure, "N" to not save. 
            Default is "N".

        Returns:
        --------
        None
        """
        
        # function implementation
        df = self.data["year"].value_counts().to_frame().reset_index()
        plt.figure(figsize=(6, 4))
        bars = plt.bar(df["year"], df["count"], color=["#ffff00", "#ff0000", "#ac25e2"])

        for bar in bars:
            yval = bar.get_height()
            plt.text(
                bar.get_x() + bar.get_width() / 2,
                yval + 0.1,
                str(int(yval)),
                ha="center",
                va="bottom",
                fontsize=7,
            )

        plt.xlabel("Years")
        plt.ylabel("No. of messages")
        plt.title("Yearly Message Stats", fontdict={"fontsize": 11})
        plt.xticks(df["year"])

        if save_figure == "Y" or save_figure == "y":
            plt.savefig("active_year.png", bbox_inches="tight")

        plt.show()

    def word_cloud(self, save_figure: str = "N") -> None:
        """
        Generate a word cloud of chat.

        Parameters:
        -----------
        save_figure : str, optional
            Specifies whether to save the figure or not. 
            "Y" to save the figure, "N" to not save. 
            Default is "N".

        Returns:
        --------
        None
        """
        
        # function implementation
        nltk.download("stopwords")

        def _preprocess_text(df):
            hindi = [chr(c) for c in range(0x0900, 0x097F)]
            temp = df[
                ~(df["message"].str.contains("<Media omitted>"))
                & ~(df["message"].str.contains("This message was deleted"))
                & ~(df["message"].str.contains("https://"))
                & ~(df.message.str.contains("|".join(hindi)))
            ]
            txt = " ".join(review for review in temp.message)
            return txt

        text = _preprocess_text(self.data)
        stop_words = set(stopwords.words("english"))
        wordcloud = WordCloud(
            width=1920,
            height=1080,
            background_color="white",
            stopwords=stop_words,
            random_state=42,
            colormap="inferno",
            collocations=False,
        ).generate(text)
        plt.figure(figsize=(6, 3))
        plt.imshow(wordcloud)

        if save_figure == "Y" or save_figure == "y":
            plt.savefig("wordcloud.png")
        plt.axis("off")
        plt.tight_layout(pad=0)
        plt.show()



import re
import pandas as pd
from ensure import ensure_annotations  # type: ignore
from typing import Union, Match


def _fetchDateAndTime(s: str) -> Union[Match[str], bool]:
    pattern = r"^([0-9]+)(\/)([0-9]+)(\/)([0-9][0-9]), ([0-9]+):([0-9][0-9]) (am|pm) - "

    result = re.match(pattern, s)
    if result:
        return result
    return False


def _findAuthor(s: str) -> Union[Match[str], bool]:
    patterns = [
        r"([\w\s]+):",  # any length name
        # r"([\w]+):",  # first name
        # r"([\w]+[\s]+[\w]+):",  # first name + last name
        # r"([\w]+[\s]+[\w]+[\s]+[\w]+):",  # first name + middle mame + last name
        r"([+]\d{1} [(]\d{3}[)] \d{3}-\d{4}):",  # +1 (205) 962-9343
        r"([+]\d{2} \d{5} \d{5}):",  # mobile number (India no.)
        r"([+]\d{2} \d{3} \d{3} \d{4}):",  # mobile number (US no.)
        r"([+]\d{2} \d{2} \d{5}-\d{4}):",  # +55 11 99224-4030
        r"([+]\d{2} \d{4} \d{3} \d{3}):",  # +91 6370 376 940
        r"([+]\d{2} \d{4} \d{4}):",  # +65 8454 7719
        r"([+]\d{2} \d{3} \d{3} \d{3}):",  # +34 543 678 231
        r"([+]\d{2} \d{4} \d{6}):",  # +21 8549 254168
        r"([+]\d{3} \d{2} \d{3} \d{4}):",  # +597 57 421 6849
        r"([+]\d{3} \d{3} \d{3} \d{4}):",  # +234 810 631 1539
        r"([\w\s]+)[\u263a-\U0001f999]+:",  # any length name and emoji
        r"[\u263a-\U0001f999]+:",  # only emoji
    ]
    pattern = "^" + "|".join(patterns)
    result = re.match(pattern, s)
    if result:
        return result
    return False


def _fetchMessages(line: str) -> tuple:
    splitLine = line.split(" - ")
    dateTime = splitLine[0]
    date, time = dateTime.split(", ")
    message = " ".join(splitLine[1:])
    if _findAuthor(message):
        splitMessage = message.split(": ")
        author = splitMessage[0]
        message = " ".join(splitMessage[1:])
    else:
        author = None

    return date, time, author, message


@ensure_annotations
def _transformData(file_path: str) -> pd.DataFrame:
    from datetime import datetime

    URLPATTERN = r"(https?://\S+)"
    MEDIAPATTERN = r"<Media omitted>"

    parsedData = []

    with open(file_path, "r", encoding="utf-8") as fp:
        fp.readline()
        # fp.readline()
        messageBuffer: list[str] = []

        date, time, author = None, None, None
        while True:
            line = fp.readline().replace("\u202f", " ")
            if not line:
                break
            line = line.strip()
            if _fetchDateAndTime(line):
                if len(messageBuffer) > 0:
                    parsedData.append(
                        [
                            date,
                            time,
                            author,
                            re.sub(r"\s+", " ", " ".join(messageBuffer)),
                        ]
                    )
                messageBuffer.clear()
                date, time, author, message = _fetchMessages(line)
                messageBuffer.append(message)
            else:
                messageBuffer.append(line)

        parsedData.append(
            [
                date,
                time,
                author,
                re.sub(r"\s+", " ", " ".join(messageBuffer)),
            ]
        )

    df = pd.DataFrame(parsedData, columns=["date", "time", "user", "message"])

    df = df.dropna()
    df = df.reset_index(drop=True)
    df["words"] = df["message"].apply(
        lambda s: len(s.split(" "))
    )  # Count number of word's in each message
    df["date"] = pd.to_datetime(df["date"], format="%d/%m/%y")
    df["media_count"] = (
        df["message"].apply(lambda x: re.findall(MEDIAPATTERN, x)).str.len()
    )
    df["url_count"] = df["message"].apply(lambda x: re.findall(URLPATTERN, x)).str.len()

    weeks = {
        0: "Monday",
        1: "Tuesday",
        2: "Wednesday",
        3: "Thrusday",
        4: "Friday",
        5: "Saturday",
        6: "Sunday",
    }
    df["day"] = df["date"].dt.weekday.map(weeks)

    # Converting 12 hour formate to 24 hour.
    lst = []
    for i in df["time"]:
        out_time = datetime.strftime(datetime.strptime(i, "%I:%M %p"), "%H:%M")
        lst.append(out_time)
    df["24H_time"] = lst

    df["hours"] = df["24H_time"].apply(lambda x: x.split(":")[0])

    df["msg_count"] = df["date"].map(df["date"].value_counts().to_dict())

    df["year"] = df["date"].dt.year

    df["mon"] = df["date"].dt.month
    months = {
        1: "Jan",
        2: "Feb",
        3: "Mar",
        4: "Apr",
        5: "May",
        6: "Jun",
        7: "Jul",
        8: "Aug",
        9: "Sep",
        10: "Oct",
        11: "Nov",
        12: "Dec",
    }
    df["month"] = df["mon"].map(months)
    df.drop("mon", axis=1, inplace=True)
    df["month_year"] = df.apply(lambda x: x["month"] + " " + str(x["year"]), axis=1)
    df["msg_count_monthly"] = df["month_year"].map(
        df["month_year"].value_counts().to_dict()
    )

    return df



from setuptools import find_packages, setup

with open("README.rst", "r", encoding="utf-8") as f:
    long_description = f.read()

__version__ = "0.0.1a6"
REPO_NAME = "chatInsights"
AUTHOR = "ronilpatil"
AUTHOR_EMAIL = "ronylpatil@gmail.com"
AUTHOR_USER_NAME = "ronylpatil"


setup(
    name=REPO_NAME,
    version=__version__,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    description=("whatsapp group chat analysis python package"),
    keywords=["chat analysis", "statistical analysis", "eda"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Operating System :: MacOS",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
        "Environment :: Console",
        "Natural Language :: English",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Communications :: Chat",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Scientific/Engineering :: Visualization",
    ],
    python_requires=">=3.9",
    project_urls={
        "Source": f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}",
        "Bug Tracker": f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}/issues",
        "Changelog": f"https://github.com/{AUTHOR_USER_NAME}/{REPO_NAME}/releases",
    },
    package_dir={"": "src"},
    packages=find_packages(where="src"),
)

# ref. for classifiers: https://pypi.org/classifiers/
